@@ -179,6 +179,37 @@ class BaseParser:
                 return self._process(match.group(1))
             return None
 
+    def find_multiline_match(self, start: str, blob: str) -> List[str]:
+        """
+        Used to find multiple lines related to a single key within the
+        WHOIS query response. Assumes the values are on a newline below
+        the key, and that a blank line separates the last value from the
+        next key or end of the text.
+
+        Example:
+        -------
+        example_blob = '''
+            Name servers:
+                 ns1.google.com
+                 ns2.google.com
+                 ns3.google.com
+                 ns4.google.com
+
+        '''
+        find_multiline_match('Name servers:\n', example_blob)
+        # would return... ['ns1.google.com', 'ns2.google.com', 'ns3.google.com', 'ns4.google.com']
+
+        :param start: a key that identifies where to begin the multiline search in blob
+        :param blob: the whois query text
+        :return: a list of values as strings
+        """
+        matches = []
+        regex_string = start + r'\s+([A-Za-z0-9\.\s]+\n\n)'
+        multiline_match = re.search(regex_string, blob, re.DOTALL | re.IGNORECASE)
+        if multiline_match:
+            matches = self._process_many(multiline_match.group(1))
+        return matches
+
     @staticmethod
     def _parse_date(date_string: str) -> Union[datetime.datetime, str]:
         """
@@ -198,9 +229,11 @@ class BaseParser:
     def _process_many(self, match: str) -> List[str]:
         if '\n' in match:
             match = match.split('\n')
-            return [self._process(m) for m in match if m]
+            matches = [self._process(m) for m in match if m]
+            return [m for m in matches if m]  # remove empty strings
         else:
-            return [self._process(match)]
+            match = self._process(match)
+            return [match] if match else []  # remove empty strings
 
     @staticmethod
     def _process(match: str) -> str:
@@ -251,6 +284,10 @@ class WhoIsParser:
             return RegexAT()
         elif tld == 'au':
             return RegexAU()
+        elif tld == 'aw':
+            return RegexAW()
+        elif tld == 'ax':
+            return RegexAX()
         elif tld == 'be':
             return RegexBE()
         elif tld == 'br':
@@ -283,6 +320,10 @@ class WhoIsParser:
             return RegexFI()
         elif tld == 'fr':
             return RegexFR()
+        elif tld == 'ge':
+            return RegexGE()
+        elif tld == 'gg':
+            return RegexGG()
         elif tld == 'gq':
             return RegexGQ()
         elif tld == 'hk':
@@ -405,7 +446,6 @@ class RegexCL(BaseParser):
 class RegexPL(BaseParser):
     _pl_expressions = {
         BaseKeys.DOMAIN_NAME: r'DOMAIN NAME: *(.+)\n',
-        BaseKeys.NAME_SERVERS: r'nameservers:(.*?)created',
         BaseKeys.REGISTRAR: r'REGISTRAR:\s*(.+)',
         BaseKeys.CREATED: r'created: *(.+)',
         BaseKeys.EXPIRES: r'option expiration date: *(.+)',
@@ -418,17 +458,10 @@ class RegexPL(BaseParser):
         self.update_reg_expressions(self._pl_expressions)
 
     def parse(self, blob: str) -> Dict[str, Any]:
-        parsed_output = {}
-        for key, regex in self.reg_expressions.items():
-            if key == BaseKeys.NAME_SERVERS:
-                match = self.find_match(regex, blob, flags=re.DOTALL | re.IGNORECASE, many=False)
-                parsed_output[BaseKeys.NAME_SERVERS] = [self._process(m) for m in match.split('\n')]
-            else:
-                many = key in self.multiple_match_keys
-                parsed_output[key] = self.find_match(regex, blob, many=many)
-                if key in self.date_keys and parsed_output.get(key, None):
-                    parsed_output[key] = self._parse_date(parsed_output.get(key))
-
+        parsed_output = super().parse(blob)
+        nameservers_match = self.find_match(r'nameservers:*(.+)\ncreated:\s', blob, flags=re.DOTALL | re.IGNORECASE)
+        if nameservers_match:
+            parsed_output[BaseKeys.NAME_SERVERS] = [self._process(m) for m in nameservers_match.split('\n')]
         return parsed_output
 
 
@@ -538,23 +571,19 @@ class RegexEU(BaseParser):
 
     def parse(self, blob: str) -> Dict[str, Any]:
         parsed_output = super().parse(blob)
-        name_servers_match = re.search(r"Name servers:\n(.+)", blob, re.DOTALL | re.IGNORECASE)
-        if name_servers_match:
-            name_servers = name_servers_match.group(1).split('\n')
-            parsed_output[BaseKeys.NAME_SERVERS] = [self._process(ns) for ns in name_servers if ns]
+        # find name servers
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match("Name servers:", blob)
         return parsed_output
 
 
 class RegexDE(BaseParser):
-    """
-    .de disclaimer (very hard to extract information from this provider):
-
-    % The DENIC whois service on port 43 doesn't disclose any information concerning
-    % the domain holder, general request and abuse contact.
-    % This information can be obtained through use of our web-based whois service
-    % available at the DENIC website:
-    % http://www.denic.de/en/domains/whois-service/web-whois.html
-    """
+    # .de disclaimer (very hard to extract information from this provider):
+    #
+    # % The DENIC whois service on port 43 doesn't disclose any information concerning
+    # % the domain holder, general request and abuse contact.
+    # % This information can be obtained through use of our web-based whois service
+    # % available at the DENIC website:
+    # % http://www.denic.de/en/domains/whois-service/web-whois.html
 
     _de_expressions = {
         BaseKeys.UPDATED: r'Changed: *(.+)',
@@ -589,9 +618,7 @@ class RegexUK(BaseParser):
             address_pieces = [m.strip() for m in address_match.group(1).split('\n') if m.strip()]
             parsed_output[BaseKeys.REGISTRANT_ADDRESS] = ", ".join(address_pieces)
         # find name servers
-        ns_match = re.search(r"Name servers: *(.+)WHOIS lookup", blob, re.DOTALL)
-        if ns_match:
-            parsed_output[BaseKeys.NAME_SERVERS] = [m.strip() for m in ns_match.group(1).split('\n') if m.strip()]
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match("Name servers:", blob)
         return parsed_output
 
 
@@ -599,10 +626,10 @@ class RegexJP(BaseParser):
     _jp_expressions = {
         BaseKeys.REGISTRANT_NAME: r'\[Registrant\] *(.+)',
         BaseKeys.CREATED: r'\[登録年月日\] *(.+)',
-        BaseKeys.EXPIRES: r'\[有効期限\] *(.+)',
+        BaseKeys.EXPIRES: r'\[(?:有効限|有効期限)\]*(.+)',
         BaseKeys.STATUS: r'\[状態\] *(.+)',
         BaseKeys.UPDATED: r'\[最終更新\] *(.+)',
-        BaseKeys.NAME_SERVERS: r'\[Name Server\]:: *(.+)'
+        BaseKeys.NAME_SERVERS: r'\[Name Server\] *(.+)'
     }
 
     def __init__(self):
@@ -612,7 +639,7 @@ class RegexJP(BaseParser):
 
     def parse(self, blob: str) -> Dict[str, Any]:
         parsed_output = super().parse(blob)
-        address_match = re.search(r"[Postal Address] *(.+)[電話番号]", blob, re.DOTALL)
+        address_match = re.search(r"\[Postal Address\]([^\[|.]+)\[\w+\](.+)", blob, re.DOTALL)
         if address_match:
             address_pieces = [m.strip() for m in address_match.group(1).split('\n') if m.strip()]
             parsed_output[BaseKeys.REGISTRANT_ADDRESS] = ", ".join(address_pieces)
@@ -633,6 +660,7 @@ class RegexAU(BaseParser):
 
 class RegexAT(BaseParser):
     _at_expressions = {
+        BaseKeys.DOMAIN_NAME: r'domain: *(.+)',
         BaseKeys.REGISTRAR: r'registrar: *(.+)',
         BaseKeys.REGISTRANT_NAME: r'personname: *(.+)',
         BaseKeys.REGISTRANT_ADDRESS: r'street address: *(.+)',
@@ -640,20 +668,13 @@ class RegexAT(BaseParser):
         BaseKeys.REGISTRANT_CITY: r'city: *(.+)',
         BaseKeys.REGISTRANT_COUNTRY: r'country: *(.+)',
         BaseKeys.UPDATED: r'changed: *(.+)',
+        BaseKeys.NAME_SERVERS: r'nserver :*(.+)'
     }
 
     def __init__(self):
         super().__init__()
         self.update_reg_expressions(self._at_expressions)
 
-    def parse(self, blob: str) -> Dict[str, Any]:
-        parsed_output = {BaseKeys.NAME_SERVERS: self.find_match(r'nserver :*(.+)', blob, many=True)}
-        for key, pattern in self.reg_expressions.items():
-            match = re.search(pattern, blob)
-            if match:
-                parsed_output[key] = self._process(match.group(1))
-        return parsed_output
-
 
 class RegexBE(BaseParser):
     _be_expressions = {
@@ -669,9 +690,7 @@ class RegexBE(BaseParser):
 
     def parse(self, blob: str) -> Dict[str, Any]:
         parsed_output = super().parse(blob)
-        ns_match = re.search(r"Name servers: *(.+)Keys: ", blob, re.DOTALL)
-        if ns_match:
-            parsed_output[BaseKeys.NAME_SERVERS] = [m.strip() for m in ns_match.group(1).split('\n') if m.strip()]
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match("Name servers:", blob)
         return parsed_output
 
 
@@ -706,9 +725,7 @@ class RegexKG(BaseParser):
 
     def parse(self, blob: str) -> Dict[str, Any]:
         parsed_output = super().parse(blob)
-        ns_match = re.search(r"Name servers in the listed order: *(.+)", blob, re.DOTALL)
-        if ns_match:
-            parsed_output[BaseKeys.NAME_SERVERS] = [m.strip() for m in ns_match.group(1).split('\n') if m.strip()]
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match("Name servers in the listed order:", blob)
         return parsed_output
 
 
@@ -1084,13 +1101,13 @@ class RegexCZ(BaseParser):
 
 class RegexHR(BaseParser):
     _hr_expressions = {
-        BaseKeys.DOMAIN_NAME: 'Domain Name: *(.+)',
-        BaseKeys.UPDATED: 'Updated Date: *(.+)',
-        BaseKeys.CREATED: 'Creation Date: *(.+)',
-        BaseKeys.EXPIRES: 'Registrar Registration Expiration Date: *(.+)',
-        BaseKeys.NAME_SERVERS: 'Name Server: *(.+)',
-        BaseKeys.REGISTRANT_NAME: 'Registrant Name:\s(.+)',
-        BaseKeys.REGISTRANT_ADDRESS: 'Registrant Street:\s*(.+)',
+        BaseKeys.DOMAIN_NAME: r'Domain Name: *(.+)',
+        BaseKeys.UPDATED: r'Updated Date: *(.+)',
+        BaseKeys.CREATED: r'Creation Date: *(.+)',
+        BaseKeys.EXPIRES: r'Registrar Registration Expiration Date: *(.+)',
+        BaseKeys.NAME_SERVERS: r'Name Server: *(.+)',
+        BaseKeys.REGISTRANT_NAME: r'Registrant Name:\s(.+)',
+        BaseKeys.REGISTRANT_ADDRESS: r'Registrant Street:\s*(.+)',
     }
 
     def __init__(self):
@@ -1332,13 +1349,8 @@ class RegexTK(BaseParser):
 
     def parse(self, blob: str) -> Dict[str, Any]:
         parsed_output = super().parse(blob)
-        # name servers
-        name_server_match = re.search(r'Domain Nameservers:(?:.*?)*(.+)Domain registered', blob,
-                                      re.IGNORECASE | re.DOTALL)
-        if name_server_match:
-            name_servers = [self._process(m) for m in name_server_match.group(1).split('\n')]
-            parsed_output[BaseKeys.NAME_SERVERS] = [ns for ns in name_servers if ns]
-
+        # handle multiline nameservers
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match('Domain nameservers:', blob)
         # a date parser exists for '%d/%m/%Y', but this interferes with the parser needed
         # for this one, which is '%m/%d/%Y', so this date format needs to be parsed separately here
         created_match = re.search(r'Domain registered: *(.+)', blob, re.IGNORECASE)
@@ -1347,7 +1359,6 @@ class RegexTK(BaseParser):
         expires_match = re.search(r'Record will expire on: *(.+)', blob, re.IGNORECASE)
         if expires_match:
             parsed_output[BaseKeys.EXPIRES] = datetime.datetime.strptime(expires_match.group(1), '%m/%d/%Y')
-
         # split domain and status
         domain_name_match = re.search(f'Domain name:(?:.*?)*(.+)Owner contact:', blob, re.IGNORECASE | re.DOTALL)
         if domain_name_match:
@@ -1357,7 +1368,6 @@ class RegexTK(BaseParser):
                 parsed_output[BaseKeys.STATUS] = [self._process(domain_and_status[1])]
             else:
                 parsed_output[BaseKeys.DOMAIN_NAME] = domain_and_status
-
         return parsed_output
 
 
@@ -1409,11 +1419,8 @@ class RegexEDU(BaseParser):
             parsed_output[BaseKeys.REGISTRANT_COUNTRY] = country
             parsed_output[BaseKeys.REGISTRANT_ADDRESS] = address
 
-        name_servers_match = re.search(r'Name Servers:(.*?)Domain', blob, re.IGNORECASE | re.DOTALL)
-        if name_servers_match:
-            name_servers = [self._process(m) for m in name_servers_match.group(1).split('\n')]
-            parsed_output[BaseKeys.NAME_SERVERS] = [ns for ns in name_servers if ns]
-
+        # handle multiline nameservers
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match('Name Servers:', blob)
         return parsed_output
 
 
@@ -1453,10 +1460,8 @@ class RegexNL(BaseParser):
 
     def parse(self, blob: str) -> Dict[str, Any]:
         parsed_output = super().parse(blob)
-        name_servers_match = re.search(r'Domain nameservers:\n(.+)Record maintained', blob, re.DOTALL | re.IGNORECASE)
-        if name_servers_match:
-            name_servers = [self._process(m) for m in name_servers_match.group(1).split('\n')]
-            parsed_output[BaseKeys.NAME_SERVERS] = [ns for ns in name_servers if ns]
+        # handle multiline nameservers
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match('Domain nameservers:', blob)
         return parsed_output
 
 
@@ -1468,3 +1473,78 @@ class RegexMA(BaseParser):
     def __init__(self):
         super().__init__()
         self.update_reg_expressions(self._ma_expressions)
+
+
+class RegexGE(BaseParser):
+    _ge_expressions = {
+        BaseKeys.REGISTRANT_NAME: r'Registrant: *(.+)',
+    }
+
+    def __init__(self):
+        super().__init__()
+        self.update_reg_expressions(self._ge_expressions)
+
+
+class RegexGG(BaseParser):
+    _gg_expressions = {
+        BaseKeys.DOMAIN_NAME: r'Domain:\n*(.+)',
+        BaseKeys.REGISTRANT_NAME: r'Registrant:\n*(.+)',
+        BaseKeys.REGISTRAR: r'Registrar:\n*(.+)',
+        BaseKeys.CREATED: r'Registered on *(.+) at',
+    }
+
+    def __init__(self):
+        super().__init__()
+        self.update_reg_expressions(self._gg_expressions)
+
+    def parse(self, blob: str) -> Dict[str, Any]:
+        parsed_output = super().parse(blob)
+        # parse created date
+        created_match = parsed_output.get('created')  # looks like 30th April 2003; need to remove day suffix
+        if created_match and isinstance(created_match, str):
+            date_string = re.sub(r'(\d)(st|nd|rd|th)', r'\1', created_match)
+            parsed_output[BaseKeys.CREATED] = datetime.datetime.strptime(date_string, '%d %B %Y')
+        # handle multiline nameservers and statuses
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match('Name servers:', blob)
+        parsed_output[BaseKeys.STATUS] = self.find_multiline_match('Domain status:', blob)
+        return parsed_output
+
+
+class RegexAW(BaseParser):
+    _aw_expressions = {
+        BaseKeys.REGISTRAR: r'Registrar:\n*(.+)',
+    }
+
+    def __init__(self):
+        super().__init__()
+        self.update_reg_expressions(self._aw_expressions)
+
+    def parse(self, blob: str) -> Dict[str, Any]:
+        parsed_output = super().parse(blob)
+        parsed_output[BaseKeys.NAME_SERVERS] = self.find_multiline_match('Domain nameservers:', blob)
+        return parsed_output
+
+
+class RegexAX(BaseParser):
+    _ax_expressions = {
+        BaseKeys.DOMAIN_NAME: r'domain\.+: *(.+)',
+        BaseKeys.REGISTRAR: r'registrar\.+: *(.+)',
+        BaseKeys.REGISTRANT_NAME: r'name\.+: *(.+)',
+        BaseKeys.REGISTRANT_COUNTRY: r'country\.+: *(.+)',
+        BaseKeys.CREATED: r'created\.+: *(.+)',
+        BaseKeys.EXPIRES: r'expires\.+: *(.+)',
+        BaseKeys.UPDATED: r'modified\.+: *(.+)',
+        BaseKeys.STATUS: r'status\.+: *(.+)',
+        BaseKeys.NAME_SERVERS: r'nserver\.+: *(.+)'
+    }
+
+    def __init__(self):
+        super().__init__()
+        self.update_reg_expressions(self._ax_expressions)
+
+    def parse(self, blob: str) -> Dict[str, Any]:
+        parsed_output = super().parse(blob)
+        addresses = self.find_match(r'address\.+: *(.+)', blob, many=True)
+        if addresses:
+            parsed_output[BaseKeys.REGISTRANT_ADDRESS] = ', '.join(addresses)
+        return parsed_output
